@@ -1,7 +1,7 @@
 import sys
 from typing import List, Dict, Any, Tuple
 import dataclasses
-from .core import Parameter, ParameterWithPath, Handler, Runtime, DefaultFactory
+from .core import Parameter, ParameterWithPath, Handler, Runtime, DefaultFactory, _empty
 from .utils import merge_parameter_trees, consolidate_parameter_tree
 from .utils import ignore_parameters
 
@@ -109,13 +109,35 @@ def add_parameters(parameters: Parameter, runtime: Runtime,
             raise RuntimeError('There was no handler registered for adding arguments')
 
 
+def consolidate_parameter_tree_with_path(parameters: Parameter):
+    parameters_with_paths = dict()
+    new_par_map = dict()
+    for p in parameters.enumerate_parameters():
+        if p.type is not None:
+            parameters_with_paths[p.full_name] = p
+
+    def _call(p, children):
+        if p.full_name not in new_par_map:
+            p2 = parameters_with_paths[p.full_name].replace(children=children)
+            new_par_map[p.full_name] = p2
+            return p2
+        else:
+            old_p = new_par_map[p.full_name]
+            old_children_names = set(ParameterWithPath(x, old_p).full_name for x in old_p.children)
+            for c in children:
+                c2 = ParameterWithPath(c, new_par_map[p.full_name])
+                if c2.full_name not in old_children_names:
+                    old_p.children.insert(0, c)
+    return parameters.walk(_call, reverse=True)
+
+
 def bind_parameters(parameters: Parameter, arguments: Dict[str, Any]):
     unknown_kwargs = dict(**arguments)
+    for p in parameters.enumerate_parameters():
+        if p.argument_name in unknown_kwargs:
+            del unknown_kwargs[p.argument_name]
 
     def bind(parameter: ParameterWithPath, children: List[Tuple[Parameter, Any]]):
-        if parameter.argument_name in unknown_kwargs:
-            del unknown_kwargs[parameter.argument_name]
-
         was_handled = False
         value = arguments
         for h in handlers:
@@ -125,23 +147,25 @@ def bind_parameters(parameters: Parameter, arguments: Dict[str, Any]):
         if was_handled:
             return parameter, value
 
-        if len(children) > 0 or parameter.name is None:
-            if dataclasses.is_dataclass(parameter.type):
-                value = parameter.type(**{p.name: x for p, x in children})
-            elif parameter.type == dict:
-                value = {p.name: x for p, x in children if p.name is not None}
+        if len(children) > 0 or parameter.name is None or dataclasses.is_dataclass(parameter.type):
+            dict_vals = {p.name: x for p, x in children if p.name is not None and x != _empty}
+            if parameter.type == dict:
+                value = dict_vals
             else:
-                value = parameter.type(**{p.name: x for p, x in children})
+                value = parameter.type(**dict_vals)
         else:
-            value = arguments[parameter.argument_name]
-            if value == parameter.default:
-                value = parameter.default_factory()
-            else:
-                was_handled = False
-                for handler in handlers:
-                    was_handled, value = handler.parse_value(parameter, value)
-                    if was_handled:
-                        break
+            value = parameter.default_factory() if parameter.default_factory is not None else _empty
+            if parameter.argument_name in arguments:
+                value = arguments[parameter.argument_name]
+                if value == parameter.default:
+                    value = parameter.default_factory()
+                else:
+                    was_handled = False
+                    for handler in handlers:
+                        was_handled, value = handler.parse_value(parameter, value)
+                        if was_handled:
+                            break
         return parameter, value
+    parameters = consolidate_parameter_tree_with_path(parameters)
     _, kwargs = parameters.walk(bind)
     return kwargs, unknown_kwargs
